#include <iostream>
#include <GL/glew.h>
#include <GL/glut.h>
#include <stdio.h>
#include <windows.h>
#include <ctime>
#include "math.h"

#include "osc/OscOutboundPacketStream.h"
#include "ip/UdpSocket.h"

#define ESC 27
#define SPACE 32
#define BACKSPACE 8

#define ADDRESS "127.0.0.1"
#define PORT 7000

static void Display(void);
static void AnimDisplay(void);

static const int NUMSTREAMS = 8;

typedef struct {
	int numPts;
	int* ptsx;
	int* ptsy;
	float r;
	float g;
	float b;
} stream;

stream* streams;
int state=0;
bool clicked;
int frame=0;
int maxframes=0;
UdpTransmitSocket transmitSocket( IpEndpointName( ADDRESS, PORT ) );

static void initStream(stream &s)
{
	s.numPts = 0;
	s.ptsx = (int*)malloc(1000*sizeof(int));
	for (int i=0; i<100; i++)
		s.ptsx[i]=0;
	s.ptsy = (int*)malloc(1000*sizeof(int));
	for (int i=0; i<100; i++)
		s.ptsy[i]=0;
	s.r=0.5+((float)(rand()%100))/200.0;
	s.g=0.5+((float)(rand()%100))/200.0;
	s.b=0.5+((float)(rand()%100))/200.0;
	//printf("%f,%f,%f\n\n",s.r,s.g,s.b);
}

static void delStream(stream &s1)
{
	free(s1.ptsx);
	free(s1.ptsy);
}

int getLongestStream()
{
	int most=0;
	for (int i=0; i<NUMSTREAMS; i++)
		if (streams[i].numPts>most)
			most=streams[i].numPts;
	return most;
}

static void NoKey(unsigned char key, int x, int y)
{
	if (key==ESC)
		exit(0);
}

static void Key(unsigned char key, int x, int y)
{
	//printf("%d\n",key);
	if (key>=49 && key<=56)
		state = key-49;
	switch(key)
	{
		case ESC:
			exit(0);
			break;
		case BACKSPACE:
			delStream(streams[state]);
			initStream(streams[state]);
			break;
		case SPACE:
			frame=0;
			maxframes = getLongestStream();
			glutDisplayFunc(AnimDisplay);
			glutIdleFunc(AnimDisplay);
			glutKeyboardFunc(NoKey);
			break;

		default:
			break;
	}
}

static void Mouse(int button, int status, int x, int y)
{
	if (status==0)
	{
		if (glutGetModifiers()==GLUT_ACTIVE_SHIFT)
			streams[state].numPts=0;
		clicked=true;
	}
	else
		clicked=false;
}

static void Motion(int x, int y)
{
	if (clicked && streams[state].numPts<1000)
	{
		streams[state].ptsx[streams[state].numPts]=x;
		streams[state].ptsy[streams[state].numPts]=y;
		streams[state].numPts++;
		//printf("%d: %d,%d - %d\n",state,x,y,streams[state].numPts);
	}
}

static void sendMessage(stream s, int snum)
{

	char buffer[1024];
	osc::OutboundPacketStream p( buffer, 1024 );
	
	p << osc::BeginMessage("/params/stream") << snum << s.ptsx[frame] << s.ptsy[frame] << osc::EndMessage;
	transmitSocket.Send( p.Data(), p.Size() );
}

static void sendStopMessage()
{
	UdpTransmitSocket transmitSocket( IpEndpointName( ADDRESS, PORT ) );

	char buffer[1024];
	osc::OutboundPacketStream p( buffer, 1024 );
	
	p << osc::BeginMessage("/params/stream") << -1 << 0 << 0 << osc::EndMessage;
	transmitSocket.Send( p.Data(), p.Size() );
}

static void drawSquare(stream s, int step, bool anim)
{
	if (anim)
	{
		glBegin(GL_LINES);
		for (int i=0; i<20; i++)
		{
			int rx=rand()%60-30;
			int ry=rand()%60-30;
			glColor4f(s.r,s.g,s.b,1.0);
			glVertex2f(s.ptsx[step],s.ptsy[step]);
			glColor4f(s.r,s.g,s.b,0);
			glVertex2f(s.ptsx[step]+rx,s.ptsy[step]+ry);
		}
		glEnd();
	}
	else
	{
		glColor4f(s.r,s.g,s.b,1.0);
		glBegin(GL_QUADS);
		glVertex2f(-4.0+s.ptsx[step],-4.0+s.ptsy[step]);
		glVertex2f(-4.0+s.ptsx[step],4.0+s.ptsy[step]);
		glVertex2f(4.0+s.ptsx[step],4.0+s.ptsy[step]);
		glVertex2f(4.0+s.ptsx[step],-4.0+s.ptsy[step]);
		glEnd();
	}
}

static void drawStream(stream s, bool anim)
{
	glColor4f(s.r,s.g,s.b,1.0);
	glBegin(GL_LINES);
	if (!anim)
		for (int i=0; i<s.numPts-1; i++)
		{
			glVertex2f(s.ptsx[i],s.ptsy[i]);
			glVertex2f(s.ptsx[i+1],s.ptsy[i+1]);
		}
	else
		for (int i=max(0,frame-20); i<frame-1; i++)
		{
			glColor4f(s.r,s.g,s.b,(i-max(0,frame-20))/20.0);
			//printf("%f,",1.0-(i-max(0,frame-20))/20.0);
			glVertex2f(s.ptsx[i],s.ptsy[i]);
			glVertex2f(s.ptsx[i+1],s.ptsy[i+1]);
		}
	glEnd();
}

static void drawContentBar(stream s)
{
	glColor4f(s.r,s.g,s.b,1.0);
	glBegin(GL_QUADS);
	glVertex2f(0,320);
	glVertex2f(s.numPts/3.125,320);
	glVertex2f(s.numPts/3.125,316);
	glVertex2f(0,316);
	glEnd();
}

static void AnimDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	for (int i=0; i<NUMSTREAMS; i++)
	{
		if (streams[i].numPts>frame)
		{
			drawSquare(streams[i],frame, true);
			drawStream(streams[i],true);
			if (frame%2==0)
				sendMessage(streams[i],i);
		}
		else
		{
			drawSquare(streams[i],streams[i].numPts-1,true);
		}
	}
	Sleep(24);
	frame++;
	if (frame>=maxframes)
	{
		glutDisplayFunc(Display);
		glutIdleFunc(Display);
		glutKeyboardFunc(Key);
		sendStopMessage();
	}
	glutSwapBuffers();
}

static void Display(void)
{
	glClear(GL_COLOR_BUFFER_BIT);
	for (int i=0; i<NUMSTREAMS; i++)
	{
		if (streams[i].numPts!=0)
		{
			drawSquare(streams[i],0,false);
			drawStream(streams[i],false);
		}
	}
	drawContentBar(streams[state]);
	Sleep(24);
	glutSwapBuffers();
}

int main(int argc, char** argv) 
{
	sendStopMessage();
	srand ( time(NULL) );
	streams = (stream*)malloc(NUMSTREAMS*sizeof(stream));
	for (int i=0; i<NUMSTREAMS; i++)
	{
		initStream(streams[i]);
	}
	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowPosition(780,64);
	glutInitWindowSize(320,320);
	glutCreateWindow("sndDraw");
	glClearColor(0.0,0.0,0.0,0.0);
	glOrtho(0, 320, 320, 0, -1, 1);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glutKeyboardFunc(Key);
	glutMouseFunc(Mouse);
	glutMotionFunc(Motion);
	glutDisplayFunc(Display);
	glutIdleFunc(Display);
	glutMainLoop();
}